import 'dart:ui';
import 'dart:async';
import 'package:flutter/material.dart' hide Image, decodeImageFromList;
import 'package:flutter/services.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/images/controllers/playercontroller.dart';
import 'package:flyinsonolite/consultation/images/vid/vidpainter.dart';
import 'package:flyinsonolite/consultation/images/vid/vidplayerevent.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';

class VidPlayer extends StatefulWidget {
  const VidPlayer(
    this.controller, {
    Key? key,
    this.width,
    this.height,
  }) : super(key: key);

  final IPlayerController controller;

  final double? width;
  final double? height;

  @override
  State<StatefulWidget> createState() => _VidPlayerState();
}

class _VidPlayerState extends State<VidPlayer> {
  Uint8List? frameBytes;
  Image? image;
  int frameIndex = 0;
  List<double> curColorFilterMatrix = <double>[
    1, 0, 0, 0, 0, // red
    0, 1, 0, 0, 0, // green
    0, 0, 1, 0, 0, // blue
    0, 0, 0, 1, 0, // alpha
  ];
  @override
  void initState() {
    widget.controller.eventHandler.addListener(onControllerEvent);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant VidPlayer oldWidget) {
    if (oldWidget.controller != widget.controller) {
      throw UnsupportedError("[VidTestPlayer] unsupport replace controller.");
    }
    super.didUpdateWidget(oldWidget);
  }

  void onControllerEvent(Object sender, VidPlayerEvent e) {
    if (!mounted) return;

    if (e is VidPlayerStatusChangeEvent) {
      if (!widget.controller.hasView) {
        updateFrame();
      }
    }
    if (e is VidPlayerFrameIndexChangeEvent) {
      onFrameChanged(e);
    }
    if (e is VidPlayerFilterChangeEvent) {
      onFilterMatrixChanged(e);
    }
  }

  void onFrameChanged(VidPlayerFrameIndexChangeEvent e) {
    loadFrame(e);
  }

  void onFilterMatrixChanged(VidPlayerFilterChangeEvent e) {
    setState(() {
      curColorFilterMatrix = e.filterMatrix;
    });
  }

  void loadFrame(VidPlayerFrameIndexChangeEvent e) async {
    frameIndex = e.index;
    final oldImage = image;
    final newImg = await _decodeImg(e.bytes);
    if (newImg != null) {
      image = newImg;
      try {
        oldImage?.dispose();
      } catch (e) {}
    }
    updateFrame();
  }

  void updateFrame() {
    if (mounted) {
      setState(() {});
    }
  }

  Future<Image?> _decodeImg(Uint8List bytes) async {
    try {
      final buffer = await ImmutableBuffer.fromUint8List(bytes);
      final descriptor = await ImageDescriptor.encoded(buffer);
      final codec = await descriptor.instantiateCodec();
      final frameInfo = await codec.getNextFrame();
      codec.dispose();
      return frameInfo.image;
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "Decode player frame error: $e");
      return null;
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget? child;
    switch (widget.controller.status) {
      case VidPlayStatus.init:
        child = Container();
        break;
      case VidPlayStatus.ready:
        child = Container();
        break;
      case VidPlayStatus.loadFail:
        child = Container();
        break;
      case VidPlayStatus.play:
      case VidPlayStatus.pause:
        child = buildFrameView(context);
        break;
      case VidPlayStatus.stop:
      case VidPlayStatus.dispose:
        child = FISText(
          "Closed",
          style: Storage.currentTheme.normalTextStyle.copyWith(
              fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
              color: Colors.black),
        );
        break;
    }
    return buildBox(context, child);
  }

  Widget buildBox(BuildContext context, Widget child) {
    return Container(
      alignment: Alignment.center,
      child: child,
    );
  }

  Widget buildFrameView(BuildContext context) {
    if (image != null) {
      final size = MediaQuery.of(context).size;
      return RepaintBoundary(
        child: CustomPaint(
          painter: VidPainter(
            image: image!,
            frameIndex: frameIndex,
            colorFilterMatrix: curColorFilterMatrix,
          ),
          isComplex: false, //是否为复杂图像（true会缓存）
          size: size,
        ),
      );
    } else {
      return Container();
    }
  }

  @override
  void dispose() {
    widget.controller.eventHandler.removeListener(onControllerEvent);
    super.dispose();
  }
}
