import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/icons/fisicons.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class TerminalImage {
  String? previewUrl;
  String? imageUrl;
  String? coverImageUrl;

  TerminalImage({
    this.previewUrl,
    this.imageUrl,
    this.coverImageUrl,
  });

  factory TerminalImage.fromJson(Map<String, dynamic> map) {
    return TerminalImage(
      previewUrl: map['PreviewUrl'],
      imageUrl: map['ImageUrl'],
      coverImageUrl: map['CoverImageUrl'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    if (previewUrl != null) map['PreviewUrl'] = previewUrl;
    if (imageUrl != null) map['ImageUrl'] = imageUrl;
    if (coverImageUrl != null) map['CoverImageUrl'] = coverImageUrl;
    return map;
  }
}

/// 所有图片的入口
class ContentImage extends StatelessWidget {
  /// 图片参数
  final RemedicalInfoDTO? remedicalInfo;

  /// 是否是测量页面
  final bool? isMeasure;

  /// 改变图片地址
  final VoidCallback? onDoubleTap;

  /// 图片单机事件
  final VoidCallback? onTap;

  /// AI 良恶性
  late final DiagnosisConclusionEnum diagnosisConclusion =
      remedicalInfo?.diagnosisConclusion ?? DiagnosisConclusionEnum.NotRequired;

  /// 图片类型
  late RemedicalFileDataTypeEnum? fileDataType;

  /// 图片地址
  late TerminalImage? terminalImage;

  /// AI测量的器官
  late final List<DiagnosisOrganEnum>? diagnosisOrgans =
      remedicalInfo?.diagnosisOrgans ?? [DiagnosisOrganEnum.Null];

  bool get isCarotidResult => remedicalInfo?.carotidResult != null;

  /// 判断是否有AI器官
  bool get hasDiagnosisOrgans => diagnosisOrgans?.isNotEmpty ?? false;

  /// 判断是否展示ai字
  bool get isShowDiagnosisOrgans => [
        DiagnosisConclusionEnum.NoObviousLesion,
        DiagnosisConclusionEnum.Benign,
        DiagnosisConclusionEnum.Malignant,
        DiagnosisConclusionEnum.BenignAndMalignant
      ].contains(diagnosisConclusion);

  /// 是否是选择状态
  final bool isPureImage;

  ///是否显示左上角序号
  final bool? ifShowIndex;

  ///图像宽度
  final double? contentWidth;

  ///图像高度
  final double? contentHeight;

  ///序号
  final int serialNo;

  ///图片文字描述
  final String? description;

  /// 图像定位描述描述
  final String? locationDescription;

  BoxFit? fit;

  ContentImage({
    super.key,
    this.terminalImage,
    this.fileDataType,
    this.remedicalInfo,
    this.isMeasure = false,
    this.onDoubleTap,
    this.onTap,
    this.isPureImage = false,
    this.ifShowIndex = true,
    this.serialNo = 0,
    this.description = '',
    this.locationDescription = '',
    this.contentHeight,
    this.contentWidth,
    this.fit,
  });

  @override
  Widget build(BuildContext context) {
    terminalImage ??= TerminalImage(
        previewUrl: remedicalInfo?.terminalImages?.previewUrl ?? '',
        imageUrl: remedicalInfo?.terminalImages?.imageUrl ?? '',
        coverImageUrl: remedicalInfo?.terminalImages?.coverImageUrl ?? '');
    fileDataType ??= remedicalInfo?.fileDataType;

    if (isPureImage) {
      return Container(
        color: Colors.black,
        child: Image.network(
          terminalImage!.coverImageUrl!,
          width: fit == null ? contentWidth ?? 300.s : null,
          height: fit == null ? contentHeight ?? 200.s : null,
          fit: fit,
          errorBuilder: ((context, error, stackTrace) {
            return FISText(i18nBook.common.error.t,
                style: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                    color: Colors.black));
          }),
        ),
      );
    } else {
      return _buildFContentImage(
        fileDataType!,
        terminalImage!.previewUrl!,
      );
    }
  }

  Widget _buildFContentImage(
    RemedicalFileDataTypeEnum fileDataType,
    String previewUrl,
  ) {
    switch (fileDataType) {
      case RemedicalFileDataTypeEnum.Image:
      case RemedicalFileDataTypeEnum.ThirdVidSingle:
      case RemedicalFileDataTypeEnum.VinnoVidSingle:
        return _buildImageCard(
          previewUrl,
        );
      case RemedicalFileDataTypeEnum.ThirdVidMovie:
      case RemedicalFileDataTypeEnum.VinnoVidMovie:
        return _buildVidMovieCard(
          previewUrl,
        );
      default:
        return FISText(i18nBook.common.error.t,
            style: Storage.currentTheme.normalTextStyle.copyWith(
                fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                color: Colors.black));
    }
  }

  Widget _buildVidMovieCard(String previewUrl) {
    return Container(
      width: 190.s,
      height: 160.s,
      color: Colors.black,
      child: Stack(
        children: [
          Center(
            child: Image.network(
              previewUrl,
              errorBuilder: ((context, error, stackTrace) {
                return FISText(i18nBook.common.error.t,
                    style: Storage.currentTheme.normalTextStyle.copyWith(
                        fontSize:
                            Storage.currentTheme.normalTextStyle.fontSize!.s,
                        color: Colors.black));
              }),
            ),
          ),
          InkWell(
            onDoubleTap: () {
              onDoubleTap?.call();
            },
            onTap: () {
              onTap?.call();
            },
            child: Center(
              child: Icon(
                Icons.play_circle_outline_rounded,
                color: Colors.white,
                size: 50.s,
              ),
            ),
          ),
          //_buildLeftAIOrCarotidLogo(),
          //_buildRigthAIText(),
          //_buildImageLocationDescription(),
          if (ifShowIndex!) _buildIndex(),
        ],
      ),
    );
  }

  Widget _buildImageCard(
    String previewUrl,
  ) {
    return InkWell(
      onDoubleTap: () {
        onDoubleTap?.call();
      },
      onTap: () {
        onTap?.call();
      },
      child: Container(
        width: 190.s,
        height: 160.s,
        color: Colors.black,
        child: Stack(
          children: [
            Center(
              child: Image.network(
                previewUrl,
                errorBuilder: ((context, error, stackTrace) {
                  return FISText(i18nBook.common.error.t,
                      style: Storage.currentTheme.normalTextStyle.copyWith(
                          fontSize:
                              Storage.currentTheme.normalTextStyle.fontSize!.s,
                          color: Colors.black));
                }),
              ),
            ),
            //_buildLeftAIOrCarotidLogo(),
            //_buildImageLocationDescription(),
            // _buildRigthAIText(),
            if (ifShowIndex!) _buildIndex(),
          ],
        ),
      ),
    );
  }

  Widget _buildIndex() {
    return Positioned(
      left: 5.s,
      child: FISText(
        (serialNo).toString(),
      ),
    );
  }

  Widget _buildLeftAIOrCarotidLogo() {
    if (isCarotidResult) {
      return Positioned(
        bottom: 5.s,
        left: 5.s,
        child: const Icon(
          //FISICons.arteria_carotis,
          FISICons.consultation,
          color: Colors.lightBlue,
        ),
      );
    }
    if (hasDiagnosisOrgans) {
      int count = 0;
      final List<Widget> aIDiagnosticOrgansIconList = [];

      for (var i = 0; i < diagnosisOrgans!.length; i++) {
        if (_ifAbdominalOrgans(diagnosisOrgans![i])) {
          //当diagnosisOrgans返回值为肝脏、胆囊、肾脏、脾脏,都只显示一个“腹部”图标
          if (count <= 0) {
            aIDiagnosticOrgansIconList.add(
              _buildAIDiagnosticOrgans(DiagnosisOrganEnum.Abdomen),
            );
          }
          count++;
        } else {
          aIDiagnosticOrgansIconList.add(
            _buildAIDiagnosticOrgans(
              diagnosisOrgans?[i],
            ),
          );
        }
      }

      return Positioned(
        bottom: 5.s,
        left: 5.s,
        child: Row(
          children: aIDiagnosticOrgansIconList,
        ),
      );
    }
    return Positioned(
      bottom: 5.s,
      left: 5.s,
      child: _buildApplication(),
    );
  }

  ///判断是否是肝脏、胆囊、肾脏、脾脏、腹部器官
  bool _ifAbdominalOrgans(DiagnosisOrganEnum diagnosisOrgan) {
    final abdominalOrgansList = [
      DiagnosisOrganEnum.Liver,
      DiagnosisOrganEnum.Cholecyst,
      DiagnosisOrganEnum.Kidney,
      DiagnosisOrganEnum.Spleen,
      DiagnosisOrganEnum.Abdomen
    ];
    return abdominalOrgansList.contains(diagnosisOrgan);
  }

  Widget _buildAIDiagnosticOrgans(DiagnosisOrganEnum? diagnosisOrganEnum) {
    switch (diagnosisOrganEnum) {
      case DiagnosisOrganEnum.Breast:
        return Icon(
          //FISICons.fis_breast,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Abdomen:
        return Icon(
          //FISICons.fis_abdomen,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );

      case DiagnosisOrganEnum.Liver:
        return Icon(
          //FISICons.fis_liver,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Cholecyst:
        return Icon(
          //FISICons.fis_gallbladder,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Kidney:
        return Icon(
          //FISICons.fis_kidney,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Spleen:
        return Icon(
          //FISICons.fis_spleen,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.CarotidArtery:
        return Icon(
          //FISICons.arteria_carotis,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );

      case DiagnosisOrganEnum.Thyroid:
        return Icon(
          //-FISICons.fis_thyroid,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Neck:
        return Icon(
          //图标待定
          //FISICons.fis_thyroid,
          FISICons.consultation,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Null:
        if (diagnosisOrgans?.length == 1) {
          //如果diagnosisOrgans只有一个值且该值为null值，就返回文字，否则返回空
          return _buildApplication();
        } else {
          return const FISText('');
        }

      default:
        return _buildApplication();
    }
  }

  ///构建描述字段
  Widget _buildApplication() {
    return FISText(
      description ?? "",
      style: TextStyle(
        fontSize: 10.s,
        overflow: TextOverflow.ellipsis,
      ),
    );
  }

  dynamic _buildAITextColor(DiagnosisConclusionEnum diagnosisConclusion) {
    switch (diagnosisConclusion) {
      case DiagnosisConclusionEnum.NoObviousLesion:
        return Colors.lightBlue;
      case DiagnosisConclusionEnum.Benign:
        return Colors.greenAccent;
      case DiagnosisConclusionEnum.Malignant:
        return Colors.orangeAccent;
      case DiagnosisConclusionEnum.BenignAndMalignant:
        return Colors.orangeAccent;
      default:
        return Colors.transparent;
    }
  }
}
