import 'package:fis_common/index.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_lib_media_rt/fis_lib_media_rt.dart';
import 'package:fis_lib_media_rt/implementations/trtc/room.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/irtmpmockrtcmanager.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/playchannelinfo.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/services/hardwaresetting.dart';
import 'package:get/get.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_listener.dart';

class MediaController {
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;
  bool _isLocalStreamPublishStarted = false;
  bool _entered = false;
  bool _isExit = false;
  bool get isRtmpMode =>
      FRTMediaConfig.serviceType == FRealTimeMediaServiceType.ntRtmp;
  late final BaseRoom _room;
  IPublisher? _publisher;
  IPlayer? _player;
  VideoDeviceOutputInfo? cameraDeviceInfo;
  MediaController(this.liveConsultationController);

  Future initAsync() async {
    ///初始化设备信息
    _initDeviceInfo();

    ///初始化房间的一些事件监听
    _initRoom();

    ///进入房间
    await _enterAsync();
  }

  Future _enterAsync() async {
    var liveConsultation = state.liveConsultation;
    var userCode = liveConsultation.value!.initiatorCode.isNotNullOrEmpty
        ? liveConsultation.value!.initiatorCode
        : liveConsultation.value!.userCode;
    _room.loadLocalMember(LocalMemberParams(
      userId: userCode ?? '',
      userSig: liveConsultation.value!.userSign,
      extParams: {},
    ));
    await _room.enter();
    Get.find<IRtmpMockRtcManager>().handleEnterRoom();
  }

  ///初始化超声机设备信息
  void _initDeviceInfo() {
    var deviceCode = liveConsultationController.consultationDetail!.deviceCode;
    if (state.liveConsultation.value?.memberLiveDatas == null) {
      return;
    }

    var memberStatus = state.liveConsultation.value!.memberLiveDatas;
    var deviceMember =
        memberStatus.firstWhereOrNull((element) => element.id == deviceCode);
    if (deviceMember != null) {
      var videoDeviceInfos = deviceMember.videoDeviceInfos ?? [];
      cameraDeviceInfo = videoDeviceInfos.firstWhereOrNull((element) =>
          element.videoDeviceSourceType == VideoDeviceSourceTypeEnum.Camera);
      cameraDeviceInfo = cameraDeviceInfo;
      state.desktopDeviceInfo = videoDeviceInfos.firstWhereOrNull((element) =>
          element.videoDeviceSourceType == VideoDeviceSourceTypeEnum.Desktop);
    }
  }

  void _initRoom() {
    var liveConsultation = state.liveConsultation;
    FRTMediaConfig.trtc.appid = liveConsultation.value!.appId;
    _room = FRealTimeMedia.createRoom(
      liveConsultation.value!.roomNo.toString(),
      FRTMediaConfig.serviceType,
    );
    _room.entered.addListener(_onEntered);
    _room.exited.addListener(_onExited);
  }

  void _onExited(Object sender, void e) {}

  void _onEntered(Object sender, void e) {
    _initPublisher();
    _initPlayer();
    if (_room is TrtcRoom) {
      (_room as TrtcRoom).events.once(TRTCCloudListener.onStreamPublish,
          (param) async {
        _isLocalStreamPublishStarted = true;
      });
    }
    if (!_entered) {
      _entered = true;
      startLocal();
    }
  }

  ///开启本地预览推流
  void startLocal() async {
    _publisher!.channels.first.readyStateChanged.addListener((sender, e) async {
      if (e) {
        WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
          Future.delayed(const Duration(milliseconds: 200), () {
            _publisher!.startLocal();
          });
        });
      }
    });
    state.playChannels.update(Storage.user.userCode!, (value) {
      value.started = true;
      return value;
    });

    /// 从超声机获取硬件设置，扬声器只能用默认的，没法设
    HardwareSetting hardwareSetting = HardwareSetting.ins;
    await hardwareSetting.setHardwareSettingsAsync();
    final String idealCameraName = hardwareSetting.cameraName;
    final String idealMicrophoneName = hardwareSetting.micName;

    //print(
    //    "⭐⭐ 预设硬件配置 idealCameraName: $idealCameraName idealMicrophoneName: $idealMicrophoneName");
    // 这里缺个事件，收到通知后可设置分辨率
    // 这里是TRTC SDK设计原因，所以要先播放才能改分辨率，是否魔改以后再说
    _publisher!.setProfile(PublisherProfile(
        resolutionMode: ResolutionMode.landscape,
        resolution: Resolutions.sd_480p,
        frameRate: 10,
        idealCameraName: idealCameraName,
        idealMicrophoneName: idealMicrophoneName));
    //_publisher!.channels.first.readyStateChanged.addListener((sender, e) async {
    //  if (e) {
    //    await _publisher!.startLocal();
    //  }
    // });
  }

  IChannel? getUserMemberPreviewChannel(ConsultationMember member) {
    if (state.playChannels.containsKey(member.id) &&
        state.playChannels[member.id]!.channel != null &&
        state.playChannels[member.id]!.started) {
      return state.playChannels[member.id]!.channel!;
    }
    return null;
  }

  void _onChannelAdded(Object s, IPlayChannel channel) async {
    channel as BasePlayChannel;
    var userId = channel.member.userId ?? '';
    if (state.playChannels.containsKey(userId)) {
      state.playChannels.update(userId, (value) {
        value.channel = channel;
        value.started = true;
        return value;
      });
    } else {
      state.playChannels[userId] = PlayChannelInfo(channel, started: true);
    }

    bool kIsCamera = cameraDeviceInfo != null &&
        cameraDeviceInfo!.videoDeviceId == userId; // 当前为摄像头流
    bool kIsDesktop = state.desktopDeviceInfo != null &&
        state.desktopDeviceInfo!.videoDeviceId == userId; // 当前为主画面流
    // 当前为摄像头流
    if (kIsCamera) {
      state.deviceCameraChannel.value = channel;
    } else if (kIsDesktop) {
      state.deviceDesktopChannel.value = channel;
      liveConsultationController.loadingController.stopLoading();
      //liveConsultationController.onDeviceChannelAdd.emit(this, channel);
      channel.onVideoInfo.addListener(_handleVideoInfoCallback);
      channel.onVideoLost.addListener(_handleVideoLostCallback);
      ConsultationMember? device = liveConsultationController
          .state.liveConsultation.value!.memberLiveDatas
          .firstWhereOrNull(
              (element) => element.memberType == LiveMemberEnum.Device);
      if (device != null) {
        if (device.mergedChannel) {
          liveConsultationController.state.isMergedMode.value = true;
          liveConsultationController.state.isNeedCropDeviceDesktop = true;
          int fullViewWidth = device.mergedVideoOutputWidth;
          int fullViewHeight = device.mergedVideoOutputHeight;
          VideoDeviceOutputInfo mainView = device.videoDeviceInfos!.firstWhere(
              (element) =>
                  element.videoDeviceSourceType ==
                  VideoDeviceSourceTypeEnum.Desktop);
          liveConsultationController.state.mainViewRectInMergedMode =
              Rect.fromLTWH(0, 0, mainView.outputWidth / fullViewWidth,
                  mainView.outputHeight / fullViewHeight);
          // 如果此时 fullViewWidth、fullViewHeight 的参数不正确，需要拿到真实回调后再次设置 controller.state.mainViewRectInMergedMode
        } else {
          liveConsultationController.state.isMergedMode.value = false;
          VideoDeviceOutputInfo mainView = device.videoDeviceInfos!.firstWhere(
              (element) =>
                  element.videoDeviceSourceType ==
                  VideoDeviceSourceTypeEnum.Desktop);
          Size realSize = selectResolution(Size(mainView.outputWidth.toDouble(),
              mainView.outputHeight.toDouble()));
          if (realSize.width.toInt() == mainView.outputWidth &&
              realSize.height.toInt() == mainView.outputHeight) {
            liveConsultationController.state.isNeedCropDeviceDesktop = false;
          } else {
            liveConsultationController.state.isNeedCropDeviceDesktop = true;
            liveConsultationController.state.mainViewRectInMergedMode =
                Rect.fromLTWH(0, 0, mainView.outputWidth / realSize.width,
                    mainView.outputHeight / realSize.height);
          }
        }
      }
    }
    channel.readyStateChanged.addListener((sender, e) {
      if (e) {
        WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
          Future.delayed(const Duration(milliseconds: 200), () {
            channel.play();
          });
        });
      }
    });
  }

  /// 处理实际视频大小回调
  void _handleVideoInfoCallback(_, Size size) {
    // print('通过实际总尺寸修正 size:$size');

    /// FIXME 此时的 controller.state.memberStatus 为旧数据，需要提前更新

    ConsultationMember device = liveConsultationController
        .state.liveConsultation.value!.memberLiveDatas
        .firstWhere((element) => element.memberType == LiveMemberEnum.Device);
    VideoDeviceOutputInfo mainView = device.videoDeviceInfos!.firstWhere(
        (element) =>
            element.videoDeviceSourceType == VideoDeviceSourceTypeEnum.Desktop);
    liveConsultationController.state.mainViewRectInMergedMode = Rect.fromLTWH(
        0,
        0,
        mainView.outputWidth / size.width,
        mainView.outputHeight / size.height);

    // 特殊情况的修正：RTMP 环境下，特殊机型例如 X1 等，接口返回的主画面尺寸不正确，需要通过实际尺寸修正
    // 参数不正确仅发生在，仅超声机画面，且 SupportRtc = false 的情况下
    // 补充，无需判定 SupportRtc，因为 SupportRtc 为 true 时，也会出现参数错乱的情况
    // http://test.ius.plus:88/mantis2/view.php?id=14598
    // if (device.videoDeviceInfos!.length == 1 && !device.supportRtc) {
    if (device.videoDeviceInfos!.length == 1) {
      if (mainView.outputWidth == size.width.toInt() &&
          mainView.outputHeight == size.height.toInt()) {
        return; // 相等则不处理
      }
      mainView.outputWidth = size.width.toInt();
      mainView.outputHeight = size.height.toInt();
      if (mainView.liveData != null) {
        mainView.liveData!.width = size.width.toInt();
        mainView.liveData!.height = size.height.toInt();
      }

      /// 需要发送虚拟【加入房间】事件
      Get.find<IRtmpMockRtcManager>().updateDevice(LiveConsultationMember(
          id: device.id,
          videoDeviceInfos: [mainView],
          memberType: device.memberType,
          mergedChannel: device.mergedChannel,
          mergedVideoOutputHeight: size.width.toInt(),
          mergedVideoOutputWidth: size.height.toInt(),
          mute: device.mute,
          videoOpend: device.videoOpend));
      print("修正接口参数，修正为：$size");
    }
  }

  /// 处理视频断流回调
  void _handleVideoLostCallback(_, bool isLost) {
    print('视频断流回调 isLost:$isLost');

    /// 将设备 channel 置空
    liveConsultationController.state.deviceDesktopChannel.value = null;
    liveConsultationController.loadingController.showRetryButton();
  }

  void _onChannelRemoved(Object s, IPlayChannel channel) {
    channel as BasePlayChannel;
    channel.onVideoInfo.removeListener(_handleVideoInfoCallback);
    channel.onVideoLost.removeListener(_handleVideoLostCallback);
    var desktopDeviceInfo = state.desktopDeviceInfo;
    var userId = channel.member.userId ?? '';
    if (cameraDeviceInfo != null && cameraDeviceInfo!.videoDeviceId == userId) {
      state.deviceCameraChannel.value = null;
    } else if (desktopDeviceInfo != null &&
        desktopDeviceInfo.videoDeviceId == userId) {
      state.deviceDesktopChannel.value = null;
      liveConsultationController.loadingController.startLoading();
    }
    if (state.playChannels.containsKey(userId)) {
      state.playChannels.update(userId, (value) {
        value.channel = null;
        value.started = false;
        return value;
      });
    } else {
      loggerAsyn(FISDeviceLogCategory.Info, 'channel remove userId:$userId');
    }
  }

  void _initPublisher() {
    if (_publisher != null) return;
    _publisher = _room.createPublisher();
    state.playChannels[Storage.user.userCode!] =
        PlayChannelInfo(_publisher!.channels.first, started: false);
  }

  void _initPlayer() {
    if (_player != null) return;
    _player = _room.createPlayer();
    _player!.channelAdded.addListener(_onChannelAdded);
    _player!.channelRemoved.addListener(_onChannelRemoved);
  }

  Size selectResolution(Size size) {
    final width = size.width;
    final height = size.height;
    if (width <= 640 && height <= 480) {
      return const Size(640, 480);
    } else if (width <= 960 && height <= 544) {
      return const Size(960, 540);
    } else if (width <= 960 && height <= 720) {
      return const Size(960, 720);
    } else if (width <= 1280 && height <= 720) {
      return const Size(1280, 720);
    } else {
      return const Size(1920, 1080);
    }
  }

  ///退出房间
  Future<void> exit() async {
    if (!_isLocalStreamPublishStarted) {
      // 未开始推流，等待3秒，如果立即退出将会无法结束 Stream 视频流
      await Future.delayed(const Duration(seconds: 3));
    }
    if (_isExit) {
      return;
    }
    try {
      await stopLocal();
      _room.exit();
      _isExit = true;
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'MediaController exit ex: $e');
    }
  }

  ///停止本地预览推流
  Future<void> stopLocal() async {
    await _publisher?.stopLocal();
    if (state.playChannels.containsKey(Storage.user.userCode)) {
      state.playChannels.update(Storage.user.userCode!, (value) {
        value.started = false;
        return value;
      });
    }
  }
}
