import 'dart:typed_data';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_lib_media_rt/fis_lib_media_rt.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/imagecompressweb.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/notifications/handlers/uploadconsultationdatahandler.dart';
import 'package:flyinsonolite/services/storageservice.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_image.dart';
import 'package:vid/us/vid_us_image_data.dart';

class RecordingController extends GetxController {
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;
  final appointmentManager = Get.find<IAppointmentManager>();
  final liveConsultationManager = Get.find<ILiveConsultationManager>();

  /// 是否需要裁剪
  bool get isNeedCrop => state.isNeedCropDeviceDesktop;

  /// 超声机主画面裁切区域
  Rect get mainUSViewCropRect => state.mainViewRectInMergedMode;

  late UploadConsultationDataHandler uploadConsultationDataHandler;

  /// 主画面key
  GlobalKey mainUSViewKey = GlobalKey();
  RecordingController(this.liveConsultationController) {
    uploadConsultationDataHandler = Get.find<UploadConsultationDataHandler>();
    uploadConsultationDataHandler.subscribe(handleUploadConsultation);
  }

  Future initCaptureImageListAsync() async {
    await _updateCaptureResultList();
  }

  @override
  void dispose() {
    uploadConsultationDataHandler.unsubscribe(handleUploadConsultation);
    super.dispose();
  }

  void handleUploadConsultation(_) {
    _updateCaptureResultList();
  }

  ///开启录制
  Future startRecordingAsync() async {
    if (state.isVidFileCapturing.value) {
      PromptBox.toast(i18nBook.realTimeConsultation.processingImage.t);
      return;
    }
    final IPlayChannel? deviceChannel = state.deviceDesktopChannel.value;
    if (deviceChannel == null) {
      PromptBox.toast(i18nBook.realTimeConsultation.noUsChannel.t);
    } else {
      if (state.isVidCapturerRecording.value) return;
      state.isVidCapturerRecording.value = true;
      final capturer = deviceChannel.createCapture();
      await capturer.initCapturer(
          isNeedCrop: isNeedCrop, cropArea: mainUSViewCropRect);
      capturer.startRecord();
      state.vidCapturerStartTime.value = DateTime.now();
    }
  }

  ///结束录制
  Future endRecordingAsync() async {
    final IPlayChannel? deviceChannel = state.deviceDesktopChannel.value;
    if (deviceChannel == null) {
      PromptBox.toast(i18nBook.realTimeConsultation.noUsChannel.t);
    } else {
      final capturer = deviceChannel.createCapture();
      if (!state.isVidCapturerRecording.value) return;
      state.isVidCapturerRecording.value = false;
      state.isVidFileCapturing.value = true;
      state.isVidFileCapturingByRecording.value = true;
      try {
        VidUsImageData vidUsImageData = await capturer.stopRecord();
        state.isVidFileCapturing.value = false;
        await _updateConsultationFilesInfoAsync(vidUsImageData, false);
      } catch (e) {
        state.isVidFileCapturing.value = false;
        PromptBox.toast(i18nBook.realTimeConsultation.recordFailed.t);
      }
    }
  }

  /// 截取一帧 vid
  Future captureOneFrameAsync() async {
    if (state.isVidFileCapturing.value) {
      PromptBox.toast(i18nBook.remedical.ImageProcessing.t);
      return;
    }
    final IPlayChannel? deviceChannel = state.deviceDesktopChannel.value;
    if (deviceChannel == null) {
      PromptBox.toast(i18nBook.realTimeConsultation.noUsChannel.t);
    } else {
      final capturer = deviceChannel.createCapture();
      state.isVidFileCapturing.value = true;
      state.isVidFileCapturingByRecording.value = false;
      await capturer.initCapturer(
          isNeedCrop: isNeedCrop, cropArea: mainUSViewCropRect);
      try {
        VidUsImageData vidUsImageData = await capturer.captureFrame();
        state.isVidFileCapturing.value = false;
        await _updateConsultationFilesInfoAsync(vidUsImageData, true);
      } catch (e) {
        PromptBox.toast(i18nBook.realTimeConsultation.captureFailed.t);
        state.isVidFileCapturing.value = false;
      }
    }
  }

  Size _countChannelSize() {
    final mainUSView = mainUSViewKey.currentContext?.findRenderObject();
    if (mainUSView != null) {
      final mainUSViewSize = mainUSView.paintBounds.size;
      final pixelRatio = Get.pixelRatio;
      final displaySize = Size(mainUSViewSize.width * pixelRatio,
          mainUSViewSize.height * pixelRatio);
      return displaySize;
    }
    return const Size(1920, 1080);
  }

  /// 上传录制的 vid
  Future<bool> _updateConsultationFilesInfoAsync(
      VidUsImageData vidUsImageData, bool isSingle) async {
    if (vidUsImageData.imageCount <= 0) {
      if (isSingle) {
        PromptBox.toast(i18nBook.realTimeConsultation.captureFailed.t);
      } else {
        PromptBox.toast(i18nBook.realTimeConsultation.recordFailed.t);
      }
      return false;
    }
    if (!state.isShowCaptureImageList.value) {
      state.isShowCaptureImageList.value = true;
    }
    state.captureImageUploadingCount++;
    state.captureImageUploadOnceSum++;
    final String suffix = DateTime.now().millisecondsSinceEpoch.toString();
    final String sourceFileName = "$suffix.vid";
    final String previewFileName = "$suffix-preview.jpg";
    final String coverFileName = "$suffix-cover.jpg";

    /// 移动端需要对 vidUsImageData 进行压缩，生成新的压缩后的 vidUsImageData
    VidUsImageData compressedImageData = vidUsImageData;
    final String sourceUrl = await jsonRpcProxy.storage
            .uploadUint8List(compressedImageData.flush(), sourceFileName) ??
        '';
    print('sourceUrl: $sourceUrl');
    Uint8List coverImageData = vidUsImageData.getImage(0).imageData;
    final String coverUrl = await jsonRpcProxy.storage
            .uploadUint8List(coverImageData, coverFileName) ??
        '';
    print('coverUrl: $coverUrl');
    final previewImage =
        await _generatePreviewImage(vidUsImageData.getImage(0).imageData);
    final String previewUrl = await jsonRpcProxy.storage
            .uploadUint8List(previewImage, previewFileName) ??
        '';
    print('previewUrl: $previewUrl');
    if (previewUrl == '' || coverUrl == '' || sourceUrl == '') {
      state.captureImageUploadingCount--;
      if (isSingle) {
        PromptBox.toast(i18nBook.realTimeConsultation.captureFailed.t);
      } else {
        PromptBox.toast(i18nBook.realTimeConsultation.recordFailed.t);
      }
      return false;
    }

    try {
      bool result =
          await liveConsultationManager.updateConsultationFilesInfoAsync(
              state.liveConsultation.value?.consultationCode ?? '',
              coverUrl,
              previewUrl,
              sourceUrl,
              isSingle);
      state.captureImageUploadingCount--;
      await _updateCaptureResultList();
      if (result) {
        await loggerAsyn(FISDeviceLogCategory.Info,
            'updateConsultationFilesInfoAsync success');
        return true;
      } else {
        await loggerAsyn(FISDeviceLogCategory.Info,
            'updateConsultationFilesInfoAsync failed');
        return false;
      }
    } catch (e) {
      state.captureImageUploadingCount--;
      if (isSingle) {
        PromptBox.toast(i18nBook.realTimeConsultation.captureFailed.t);
      } else {
        PromptBox.toast(i18nBook.realTimeConsultation.recordFailed.t);
      }
      return false;
    }
  }

  /// 更新捕获结果列表
  Future<void> _updateCaptureResultList() async {
    if (state.captureImageUploadingCount > 0) return;
    state.captureImageUploadOnceSum.value = 0;
    final List<ConsultationImagesDTO> consultationImageFolders =
        await appointmentManager.getConsultationImagesAsync(
            state.liveConsultation.value?.consultationCode ?? '');
    if (consultationImageFolders.isEmpty) return;
    List<ConsultationFileDTO> captureImageList = [];
    for (var consultationImageFolder in consultationImageFolders) {
      if (consultationImageFolder.screenshotList != null) {
        captureImageList.addAll(consultationImageFolder.screenshotList!);
      }
    }
    state.captureImageList.value = captureImageList;
  }

  /// 传入 jpegBytes，绘制到canvas生成缩略图，返回 jpegBytes
  Future<Uint8List> _generatePreviewImage(Uint8List jpegBytes) async {
    //// Web 端需要自己扩展压缩方法
    var result = await compressImage(jpegBytes, 80, 200);
    return result;
  }

  /// 压缩 vid
  Future<VidUsImageData> _compressVid(VidUsImageData vidUsImageData) async {
    final int imageCount = vidUsImageData.imageCount;
    final double fps = vidUsImageData.probe.frameRate;
    final VidUsImageData compressedImageData =
        VidUsImageData.empty(imageCount, VidUsImageFormat.Jpeg);
    for (int i = 0; i < imageCount; i++) {
      final VidUsImage vidUsImage = vidUsImageData.getImage(i);
      final Uint8List compressedJpegBytes =
          await compressImage(vidUsImage.imageData, 50, 1080);
      compressedImageData.addJpegImage(
          compressedJpegBytes, vidUsImage.width, vidUsImage.height);
    }
    compressedImageData.setThirdPartFPS(fps.toInt());
    return compressedImageData;
  }
}
