import 'package:fis_common/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/emergency/controllers/emergencypreparednesscontroller.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/helpers/dialoghelper.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:get/get.dart';

class ChoosePatientController extends GetxController {
  RxList<PatientInfo> patientList = RxList([]);
  Rx<PatientInfo?> selectedPatient = Rx(null);
  var keyword = "".obs;
  final int _pageSize = 20;
  int _pageIndex = 1;
  int totalCount = 0;
  final _appointmentManager = Get.find<IAppointmentManager>();
  bool isEmergencyMode = false;
  Future _findPatientListAsync() async {
    final result = await _appointmentManager.findConsultationPatientAsync(
        keyword.value.trim(), _pageIndex, _pageSize);
    _handleConsultationPatient(result.pageData ?? []);
    totalCount = result.totalCount;
    _pageIndex++;
    if (!isEmergencyMode) {
      var applyConsultationController = Get.find<ApplyConsultationController>();
      selectedPatient.value = applyConsultationController.currentPatient;
    } else {
      var emergencyPreparednessController =
          Get.find<EmergencyPreparednessController>();
      selectedPatient.value = emergencyPreparednessController.currentPatient;
    }
  }

  Future initializeAsync(bool emergencyMode) async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      patientList.clear();
      isEmergencyMode = emergencyMode;
      _pageIndex = 1;
      await _findPatientListAsync();
    } catch (e) {
      await handlerErrorAsync(
          e, 'ChoosePatientController initializeAsync error: $e');
    } finally {
      EasyLoading.dismiss();
    }
  }

  Future searchPatientAsync() async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      patientList.clear();
      _pageIndex = 1;
      await _findPatientListAsync();
      if (patientList.isEmpty && !keyword.value.isNullOrWhiteSpace) {
        DialogHelper.showConfirmDialog(
          Get.context!,
          i18nBook.remedical.hint4patientNotFound.translate([keyword.value]),
          () async {
            await createPatientAsync(false);
            await searchPatientAsync();
          },
        );
      }
    } catch (e) {
      await handlerErrorAsync(
          e, 'ChoosePatientController searchPatientAsync error: $e');
    } finally {
      EasyLoading.dismiss();
    }
  }

  /// 根据名字快捷创建病人
  Future<void> createPatientAsync(bool needAutoUpdate) async {
    if (keyword.trim().isEmpty) {
      PromptBox.toast(
          i18nBook.realTimeConsultation.pleaseEnterCreatedPatientName.t);
      return;
    }
    await _createPatientAsync(needAutoUpdate);
  }

  Future _createPatientAsync(bool needAutoUpdate) async {
    try {
      EasyLoading.show(status: i18nBook.common.submitting.t);
      var newPatient =
          await _appointmentManager.createPatientByUnregisteredAsync(
        keyword.value.trim(),
      );
      if (needAutoUpdate) {
        var newPatientInfo = _convertDtoToPatinetInfo(newPatient!);
        if (!isEmergencyMode) {
          var applyConsultationController =
              Get.find<ApplyConsultationController>();
          applyConsultationController.updatePatientInfo(newPatientInfo);
        } else {
          var emergencyPreparednessController =
              Get.find<EmergencyPreparednessController>();
          emergencyPreparednessController.updatePatientInfo(newPatientInfo);
        }
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, "choose patient controller create patient error");
    } finally {
      EasyLoading.dismiss();
      Get.close(1);
    }
  }

  Future searchMorePatientAsync() async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      await _findPatientListAsync();
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'ChoosePatientController searchMorePatientAsync error: $e');
    } finally {
      EasyLoading.dismiss();
    }
  }

  String _getPatientName(List<DataItemDTO> patientData) {
    var name = _getPatientValue(patientData, 'Name');
    if (name.isEmpty) {
      name = _getPatientValue(patientData, 'AnimalInfoName');
    }
    return name;
  }

  /// 处理病人参数
  String _getPatientValue(List<DataItemDTO> patientData, String key) {
    return patientData
            .firstWhereOrNull(
              (element) => element.key == key,
            )
            ?.value ??
        '';
  }

  /// 处理病人数据
  void _handleConsultationPatient(
      List<ClientPatientInfoBaseDTO> clientPatients) {
    for (int i = 0; i < clientPatients.length; i++) {
      patientList.add(_convertDtoToPatinetInfo(clientPatients[i]));
    }
  }

  PatientInfo _convertDtoToPatinetInfo(
      ClientPatientInfoBaseDTO patientInfoBaseDTO) {
    final List<DataItemDTO> patientData = patientInfoBaseDTO.patientData ?? [];
    final String name = _getPatientName(patientData);
    final String age = _getPatientValue(patientData, 'Age');
    final String sex = _getPatientValue(patientData, 'Sex');
    final String phone = _getPatientValue(patientData, 'Phone');
    final String identityCard = _getPatientValue(patientData, 'IdentityCard');
    final String insuranceCode = _getPatientValue(patientData, 'InsuranceCode');
    final String birthday = _getPatientValue(patientData, 'Birthday');
    final String ageYear = _getPatientValue(patientData, 'AgeYear');
    final String ageMonth = _getPatientValue(patientData, 'AgeMonth');
    final String ageWeek = _getPatientValue(patientData, 'AgeWeek');
    final String ageDay = _getPatientValue(patientData, 'AgeDay');
    final String ageUnits = _getPatientValue(patientData, 'AgeUnits');
    final String anamnesis = _getPatientValue(patientData, 'Anamnesis');
    final String headImgUrl = _getPatientValue(patientData, 'HeadImgUrl');
    final String species = _getPatientValue(patientData, 'Species');
    final String breed = _getPatientValue(patientData, 'Breed');
    var patientInfo = PatientInfo(
        isValid: patientInfoBaseDTO.isValid,
        createTime: patientInfoBaseDTO.createTime ?? DateTime.now(),
        patientCode: patientInfoBaseDTO.patientCode ?? '',
        name: name,
        identityCard: identityCard,
        insuranceCode: insuranceCode,
        age: age,
        sex: sex,
        phone: phone,
        birthday: birthday,
        ageYear: ageYear,
        ageMonth: ageMonth,
        ageWeek: ageWeek,
        ageDay: ageDay,
        ageUnits: ageUnits,
        anamnesis: anamnesis,
        headImgUrl: headImgUrl,
        species: species,
        breed: breed);

    return patientInfo;
  }
}
