import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/filters/controllers/filterdrawercontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/emergency/views/emergencypreparednessdialog.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/records/models/consultationviewmodel.dart';
import 'package:get/get.dart';

class ConsultationListController extends GetxController {
  final int _pageSize = 30;
  int _pageIndex = 1;

  final _appointmentManager = Get.find<IAppointmentManager>();
  final consultationFilter = Get.find<ConsultationFilter>();
  List<OrganizationBaseDTO> _parentOrganizationList = [];
  RxList<ConsultationViewModel> consultationList = RxList([]);
  Rx<ConsultationViewModel?> selectedItem = Rx(null);
  int totalCount = 0;

  Future updateSelectedConsultationAsync(
      ConsultationViewModel consultationViewModel) async {
    if (selectedItem.value == null ||
        selectedItem.value?.consultationCode !=
            consultationViewModel.consultationCode) {
      selectedItem.value = consultationViewModel;
      var detailController = Get.find<ConsultationDetailController>();
      await detailController.updateDataAsync();
    }
  }

  Future<List<OrganizationBaseDTO>> findParentOrganizationsAsync() async {
    if (_parentOrganizationList.isEmpty) {
      _parentOrganizationList =
          await _appointmentManager.findParentOrganizationsAsync();
    }

    return _parentOrganizationList;
  }

  Future refreshAsync() async {
    consultationList.clear();
    selectedItem.value = null;
    var detailController = Get.find<ConsultationDetailController>();
    await detailController.onTabChangedAsync(0);
    _pageIndex = 1;
    if (!onFilterDateComparison()) {
      return;
    }
    EasyLoading.show(status: i18nBook.common.loading.t);
    await _findConsultationsByPageAsync();
    EasyLoading.dismiss();
    if (!consultationList.any((element) =>
        element.consultationCode == selectedItem.value?.consultationCode)) {
      selectedItem.value = null;
    }
    await detailController.onTabChangedAsync(0);
  }

  Future resetSearchFilterAsync() async {
    consultationFilter.reset();
    var filterDrawerController = Get.find<FilterDrawerController>();
    filterDrawerController.reset();
    await refreshAsync();
  }

  Future initilizeAsync() async {
    if (Storage.user.token.isEmpty) {
      return;
    }
    EasyLoading.show(status: i18nBook.common.loading.t);
    await refreshAsync();
    EasyLoading.dismiss();
  }

  Future searchMoreAsync() async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    await _findConsultationsByPageAsync();
    EasyLoading.dismiss();
  }

  Future _findConsultationsByPageAsync() async {
    try {
      consultationFilter.expertCodes.removeWhere((element) => element.isEmpty);
      consultationFilter.expertOrganizationCodes
          .removeWhere((element) => element.isEmpty);
      final result = await _appointmentManager.findConsultationsByPageAsync(
          consultationFilter.consultationQueryType, _pageIndex, _pageSize,
          keyword: consultationFilter.keyword,
          startDate: consultationFilter.startDate.value,
          endDate: consultationFilter.endDate.value,
          consultationStatus: consultationFilter.consultationStatus,
          evaluateGrade: consultationFilter.evaluateGrade,
          expertCodes: consultationFilter.expertCodes,
          expertOrganizationCodes: consultationFilter.expertOrganizationCodes,
          patientDiseases: consultationFilter.diseaseName,
          patientPrimaryDiagnosis: consultationFilter.primaryDiagnosis,
          language: i18nBook.locale.toCodeString('-'));
      final List<ConsultationPageDTO> resultPageData = result.pageData ?? [];
      for (int i = 0; i < resultPageData.length; i++) {
        var consultationViewModel = ConsultationViewModel(
          consultationCode: resultPageData[i].consultationCode ?? '',
          patientName: resultPageData[i].patientName ?? '',
          gender: resultPageData[i].sex,
          age: resultPageData[i].patientAge,
          phone: resultPageData[i].phone,
          applyOrganizationName: resultPageData[i].applyOrganizationName,
          consultationTime: resultPageData[i].consultationTime?.toLocal(),
          consultationTimeEnd: resultPageData[i].consultationTimeEnd?.toLocal(),
          expertUserName:
              resultPageData[i].expertUserName ?? i18nBook.homePage.notYet.t,
          consultationEvaluate: resultPageData[i].consultationEvaluate,
        );
        consultationViewModel.state.value =
            resultPageData[i].consultationStatus;
        if (!consultationList.any((element) =>
            element.consultationCode ==
            consultationViewModel.consultationCode)) {
          consultationList.add(consultationViewModel);
        }
      }

      totalCount = result.totalCount;
      _pageIndex++;
    } catch (e) {
      await handlerErrorAsync(e,
          "ConsultationListController findConsultationsByPageAsync exception:$e");
      printError(
          info:
              "ConsultationListController findConsultationsByPageAsync exception:$e");
    }
  }

  /// 校验表格时间筛选 开始时间不能大于结束时间
  bool onFilterDateComparison() {
    if (consultationFilter.startDate.value != null &&
        consultationFilter.endDate.value != null) {
      DateTime startDate = consultationFilter.startDate.value!;
      DateTime endDate = consultationFilter.endDate.value!;
      if (startDate.isAfter(endDate)) {
        PromptBox.toast(i18nBook.errorCodes.errorCode4022.t);
        return false;
      }
    }

    return true;
  }

  Future logoutAsync() async {
    await Storage.user.handleLogOutAsync();
  }

  void updateItem(ConsultationDetailDTO consultationDetail) {
    var item = consultationList.firstWhereOrNull((element) =>
        element.consultationCode == consultationDetail.consultationCode);
    if (item != null) {
      item.consultationTime = consultationDetail.consultationTime;
      item.consultationTimeEnd = consultationDetail.consultationTimeEnd;
      item.state.value = consultationDetail.consultationStatus;
    }
  }

  Future startEmergencyAsync() async {
    try {
      final liveConsultationManager = Get.find<ILiveConsultationManager>();
      var datas = await liveConsultationManager.queryEmergencyDataAsync();
      if (datas.deviceInfoList!.isNotEmpty && datas.expertList!.isNotEmpty) {
        var emergencySetting =
            await liveConsultationManager.getEmergencySettingAsync();
        Get.dialog(EmergencyPreparednessDialog(datas, emergencySetting),
            barrierDismissible: false);
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'ConsultationListController startEmergencyAsync error: $ex');
    }
  }

  Future switchToLocalPatientAsync(String consultationCode) async {
    try {
      var fisLiveConsultationDTO = FISSwitchToLocalPatientRequestDTO(
          consultationCode: consultationCode, token: Storage.user.token);
      var result = await jsonRpcProxyForFISLib.consultation
          .switchToLocalPatientFromFlutter(fisLiveConsultationDTO);
      if (!result.isSuccess) {
        await loggerAsyn(FISDeviceLogCategory.Error,
            "ConsultationListController switchToLocalPatientAsync Fail:$result.failMessage");
        PromptBox.toast(
            "${i18nBook.errorCodes.errorCodeUN.t}:  ConsultationListController switchToLocalPatientAsync Fail: ${result.failMessage}");
      }
    } catch (e) {
      printError(
          info:
              "ConsultationListController switchToLocalPatientAsync exception:$e");
      await handlerErrorAsync(e,
          "ConsultationListController switchToLocalPatientAsync exception:$e");
    }
  }
}

class ConsultationFilter {
  QueryConsultationStatusEnum? consultationStatus;
  QueryEvaluateGradeEnum? evaluateGrade;
  ConsultationQueryTypeEnum consultationQueryType =
      ConsultationQueryTypeEnum.All;
  Rx<DateTime?> startDate = Rx(null);
  Rx<DateTime?> endDate = Rx(null);
  String? keyword;
  String? diseaseName;
  String? primaryDiagnosis;

  List<String> expertCodes = [];
  List<String> expertOrganizationCodes = [];

  ConsultationFilter() {
    var now = DateTime.now();
    startDate.value = DateTime(now.year, now.month, now.day - 30, 0, 0);
    endDate.value = DateTime(now.year, now.month, now.day + 30, 0, 0);
  }

  void reset() {
    var now = DateTime.now();
    keyword = '';
    consultationQueryType = ConsultationQueryTypeEnum.All;
    consultationStatus = null;
    startDate.value = DateTime(now.year, now.month, now.day - 30, 0, 0);
    endDate.value = DateTime(now.year, now.month, now.day + 30, 0, 0);
    expertCodes.clear();
    expertOrganizationCodes.clear();
    diseaseName = null;
    primaryDiagnosis = null;
  }
}
