import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

/// 可选的放置位置
enum DisplayPosition {
  top,
  right,
  bottom,
  left,
  topRight,
  bottomRight,
  bottomLeft,
  topLeft,
}

/// 支持自定义的纯文本提示框组件
class TextTooltip extends StatefulWidget {
  const TextTooltip({
    super.key,
    required this.message,
    required this.child,
    required this.height,
    required this.padding,
    required this.textStyle,
    this.position = DisplayPosition.top,
    this.margin = const EdgeInsets.all(0),
    this.offset = const Offset(0, 0),
    this.decoration,
  }) : super();
  final String message;
  final Widget child;
  final double height;
  final DisplayPosition position;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final Offset offset;
  final Decoration? decoration;
  final TextStyle textStyle;

  @override
  State<TextTooltip> createState() => TextTooltipState();
}

class TextTooltipState extends State<TextTooltip> {
  OverlayEntry? _entry;
  Timer? _timer;
  bool _isShow = false;

  /// 需要靠Bottom定位的情况
  static const List<DisplayPosition> NEED_BOTTOM = [
    DisplayPosition.top,
    DisplayPosition.topRight,
    DisplayPosition.topLeft,
  ];

  /// 需要靠Right定位的情况
  static const List<DisplayPosition> NEED_RIGHT = [
    DisplayPosition.left,
    DisplayPosition.topLeft,
    DisplayPosition.bottomLeft,
  ];

  /// 将容器扩宽来适应上下显示时的文本居中
  static const List<DisplayPosition> NEED_ENLARGE_WIDTH = [
    DisplayPosition.top,
    DisplayPosition.bottom,
  ];

  double get _maxMessageWidth =>
      widget.message.length * widget.textStyle.fontSize!;
  double get _paddingWidth => widget.padding.horizontal;
  double get _marginWidth => widget.margin.horizontal;
  double get _enlargeWidth => _maxMessageWidth + _paddingWidth + _marginWidth;

  /// 显示 Tooltip
  void _showTooltip() {
    _isShow = true;
    _createNewEntry();
    _timer?.cancel();
    _timer = Timer(const Duration(seconds: 3), () {
      _hideTooltip();
    });
  }

  /// 隐藏 Tooltip
  void _hideTooltip() {
    _entry?.remove();
    _entry = null;
    _timer?.cancel();
    _isShow = false;
  }

  /// 创建新的 OverlayEntry
  void _createNewEntry() {
    final OverlayState overlayState = Overlay.of(
      context,
      debugRequiredFor: widget,
    );

    final RenderBox target = context.findRenderObject()! as RenderBox;
    final RenderBox targetConatiner =
        overlayState.context.findRenderObject() as RenderBox;
    final Offset targetOffset = target.localToGlobal(
      target.size.center(Offset.zero),
      ancestor: overlayState.context.findRenderObject(),
    );
    final Offset overlayOffset = _countOffset(
        targetConatiner.size, target.size, targetOffset, widget.position);
    final Widget overlay = _buildOverlay(overlayOffset, widget.position);
    _entry = OverlayEntry(builder: (BuildContext context) => overlay);
    overlayState.insert(_entry!);
  }

  /// 创建 Overlay 组件
  Widget _buildOverlay(Offset offset, DisplayPosition position) {
    return Directionality(
      textDirection: Directionality.of(context),
      child: Positioned.directional(
        top: NEED_BOTTOM.contains(position) ? null : offset.dy,
        start: NEED_RIGHT.contains(position) ? null : offset.dx,
        bottom: NEED_BOTTOM.contains(position) ? offset.dy : null,
        end: NEED_RIGHT.contains(position) ? offset.dx : null,
        textDirection: Directionality.of(context),
        child: SizedBox(
          width: NEED_ENLARGE_WIDTH.contains(position) ? _enlargeWidth : null,
          child: Center(
            child: UnconstrainedBox(
              child: Transform(
                transform: Matrix4.translationValues(
                  widget.offset.dx,
                  widget.offset.dy,
                  0,
                ),
                child: Container(
                  decoration: widget.decoration ??
                      BoxDecoration(
                        color: Colors.black,
                        borderRadius: BorderRadius.all(Radius.circular(4.s)),
                      ),
                  height: widget.height,
                  padding: widget.padding,
                  margin: widget.margin,
                  child: Center(
                    child: DefaultTextStyle(
                      style: TextStyle(
                          decoration: TextDecoration.none,
                          fontFamily: Storage.currentTheme.fontFamily),
                      child: FISText(widget.message, style: widget.textStyle),
                    ),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  /// 计算位置布局
  Offset _countOffset(Size containerSize, Size targetSize, Offset targetOffset,
      DisplayPosition position) {
    final double x = targetOffset.dx;
    final double y = targetOffset.dy;
    final double w = targetSize.width;
    final double h = targetSize.height;
    final double cw = containerSize.width;
    final double ch = containerSize.height;
    switch (position) {
      case DisplayPosition.left:
        return Offset(cw - x + w / 2, y - widget.height / 2);
      case DisplayPosition.top:
        return Offset(x - _enlargeWidth / 2, ch - y + h / 2);
      case DisplayPosition.right:
        return Offset(x + w / 2, y - widget.height / 2);
      case DisplayPosition.bottom:
        return Offset(x - _enlargeWidth / 2, y + h / 2);
      case DisplayPosition.topRight:
        return Offset(x + w / 2, ch - y + h / 2);
      case DisplayPosition.bottomRight:
        return Offset(x + w / 2, y + h / 2);
      case DisplayPosition.bottomLeft:
        return Offset(cw - x + w / 2, y + h / 2);
      case DisplayPosition.topLeft:
        return Offset(cw - x + w / 2, ch - y + h / 2);
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (_isShow) {
          _hideTooltip();
        } else {
          _showTooltip();
        }
      },
      child: Container(
        child: widget.child,
      ),
    );
  }

  @override
  void dispose() {
    _hideTooltip();
    super.dispose();
  }
}
