import 'dart:convert';
import 'dart:typed_data';
import 'package:fis_common/env/env.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';

/// 文件存储
abstract class FileStorage {
  FileStorage({
    required this.storageName,
  });

  /// 文件名
  final String storageName;

  /// 保存文件
  Future<bool> setFile(String fileName, Uint8List file) async {
    try {
      return await _setFile(fileName, file);
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "Text storage save fail, platform: ${FPlatform.current}. $e");
      return false;
    }
  }

  /// 获得文件
  Future<Uint8List?> getFile(String fileName) async {
    try {
      return await _getFile(fileName);
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "File storage fetch fail, platform: ${FPlatform.current}. $e");
      return null;
    }
  }

  Future<bool> _setFile(String fileName, Uint8List file);

  Future<Uint8List?> _getFile(String fileName);

  /// 创建实例
  ///
  /// [storageName] 文件名
  static FileStorage createInstance(String storageName) {
    return _FileStoragePureWeb(storageName);
  }
}

///纯web端的文件缓存
class _FileStoragePureWeb extends FileStorage {
  _FileStoragePureWeb(String storageName) : super(storageName: storageName);

  @override
  Future<Uint8List?> _getFile(String fileName) async {
    //目前web端的持久化缓存必要性并不高
    //另外，web端没有持久化文件缓存的方案
    return null;
  }

  @override
  Future<bool> _setFile(String fileName, Uint8List file) async {
    //目前web端的持久化缓存必要性并不高
    //另外，web端没有持久化文件缓存的方案
    return false;
  }
}

/// 基于Base64字符串的壳子下文件导出
class TextMediaFileExporterForShell {
  static Future<bool> exportFile(List<int> files, String storageName) async {
    final platform = FPlatform.current;
    if (platform == FPlatformEnum.webOnMac ||
        platform == FPlatformEnum.webOnWin) {
      var data = base64Encode(files);
      return await jsonRpcProxy.platform.setBytesToExport(data, storageName);
    } else {
      //Web端不需要特殊处理，手机端也有native的处理，因此直接返回true
      return true;
    }
  }
}
