import 'dart:async';
import 'package:fis_common/extensions/string_formatter.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/foundation.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/connection.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/login/models/loginheartbeat.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iserverconfigmanager.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/notifications/notifications.dart';
import 'package:get/get.dart';

class User {
  String _token = '';
  String? _userCode = '';

  ///登录后显示的名称，已安优先级排序获得
  String? _displayName;
  List<String> _features = [];
  ServerSettingResult? _serverSetting;
  LoginHeartbeat? _loginHeartbeat;
  String? _organizationCode;
  String? _organizationName;
  String? _signature = '';
  Timer? _fisCheckTimer;
  bool _passwordExpired = false;

  /// token
  String get token => _token;
  List<String> get features => _features;
  String? get displayName => _displayName;
  String? get userCode => _userCode;
  TransactionStatusEnum get liveProtocol => _serverSetting != null
      ? _serverSetting!.liveProtocolType
      : TransactionStatusEnum.TRTC;
  bool get passwordExpired => _passwordExpired;

  /// 用户签名
  String? get signature => _signature;

  ///所属医院Code
  String? get organizationCode => _organizationCode;

  ///所属医院名称
  String? get organizationName => _organizationName;
  bool hasFeature(String key) => features.contains(key);

  /// 专家助理才有的权限 申请人权限
  bool get isApplyUserField => hasFeature('ApplyUserField');

  ///会诊心跳
  int? get liveConsultationRateSeconds =>
      _serverSetting?.liveConsultationRateSeconds;

  ///设置用户 Token
  void setToken(String token) {
    _token = token;
  }

  /// 处理登录事件
  Future handleLoginAsync(
      {required String token, required bool isPasswordExpired}) async {
    _token = token;
    _passwordExpired = isPasswordExpired;
    var userInfo = await jsonRpcProxy.user.getUserInfoAsync(GetUserInfoRequest(
      token: token,
    ));
    _userCode = userInfo.userCode;
    _signature = userInfo.signature;
    _organizationCode = userInfo.rootOrganizationCode;
    _organizationName = userInfo.rootOrganizationName;
    //_rememberPwd = userInfo.rem;
    //_password = userInfo.pa;
    _displayName = userInfo.displayName;
    await _fetchFeaturesAsync();
    await _informUserStatusAsync(true);
    await _getDeviceShortCode();
    await _autoBindDeviceAsync();
    //await Store.persistent();
  }

  /// 处理登出事件
  Future<void> handleLogOutAsync({bool goToLoginPage = true}) async {
    try {
      Storage.isLogouting = true;
      cancelFISCheckTimer();
      await autoHangUpAsync();
      if (_token.isNotEmpty) {
        await jsonRpcProxy.login.loginOutAsync(TokenRequest(
          token: _token,
        ));
      }
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error, 'User handelLogOut ex: $e');
    }
    await _informUserStatusAsync(false);
    _token = '';
    _passwordExpired = false;
    _features = [];
    Storage.configurationFromerver = null;
    Storage.isConsultating.value = false;
    Storage.isUseFISForConsultation.value = false;
    notificationCenter.stop();
    _loginHeartbeat?.stop();
    _loginHeartbeat = null;
    Storage.isLogouting = false;
    if (goToLoginPage) {
      await Get.offAllNamed(Storage.loginRoute);
    }
  }

  void cancelFISCheckTimer() {
    if (_fisCheckTimer != null && _fisCheckTimer!.isActive) {
      _fisCheckTimer?.cancel();
    }
  }

  Future<void> _fetchFeaturesAsync() async {
    _features = [];
    try {
      var items = await jsonRpcProxy.user.getUserFeatureListAsync(
        TokenRequest(token: token),
      );
      var featureKeys = items
          .where((e) => e.uniqueCode != null && e.uniqueCode!.isNotEmpty)
          .map((e) => e.uniqueCode!);
      _features = List.unmodifiable(featureKeys);
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "User fetch features error: $e");
    }
  }

  Future handleAfterLoginAsync({WSType type = WSType.Main}) async {
    try {
      await initNotificationConfigAsync(type: type);
      if (type == WSType.Main) {
        //第二屏无需心跳即检查FIS状态
        // 开启登录心跳
        _startLoginHeartbeat();

        _fisCheckTimer = Timer(const Duration(seconds: 3), () {
          fisCheck();
        });
      }

      /// 开启消息中心后，更新移动端系统通知
      //_updateMobileLocalNotification();

      /// 处理移动端打开应用携带的payload
      //_loadLaunchPayload();
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'Handle user state after logon error $e');
    }
  }

  ///初始化通知中心
  Future initNotificationConfigAsync({WSType type = WSType.Main}) async {
    await _updateServerSettingAsync();
    _updateNotificationHost(type: type);
    // 开启消息中心
    notificationCenter.start();
  }

  /// 更新服务配置
  Future _updateServerSettingAsync() async {
    try {
      var serverConfigManager = Get.find<IServerConfigManager>();
      var result = await serverConfigManager.getAllServerConfigAsync();
      _serverSetting = result;
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'User _updateServerSettingAsync ex: $e');
    }
  }

  /// 更新通知中心连接地址
  void _updateNotificationHost({WSType type = WSType.Main}) {
    final notificationUrl = _serverSetting?.notificationUrl ?? '';
    final host = StringFormatter.format(
        notificationUrl, [Storage.user.token, type.index]);

    /// Debug 用的 host
    /// 可搭配 http://git.ius.plus:88/gavin.chen/mock_websocket_server 使用
    var mockHost = ''; //'ws://127.0.0.1:8765';
    if (kDebugMode && mockHost.isNotEmpty) {
      notificationCenter.setHost(mockHost);
    } else {
      notificationCenter.setHost(host);
    }
  }

  void _startLoginHeartbeat() {
    if (_loginHeartbeat == null) {
      final rateSeconds = _serverSetting?.heartRateSeconds ?? 30;
      final interval = rateSeconds * 1000 - 500; // 设置500ms的buffer，防止超时请求
      _loginHeartbeat = LoginHeartbeat(token: token, interval: interval);
    }
    _loginHeartbeat!.start();
  }

  Future<void> autoHangUpAsync() async {
    var liveConsultationManager = Get.find<ILiveConsultationManager>();
    try {
      if (liveConsultationManager.currentConsultationCode.isNotEmpty &&
          Storage.isConsultating.value) {
        liveConsultationManager.onCloseConsultation.emit(
            this,
            LiveConsultation(
                consultationCode:
                    liveConsultationManager.currentConsultationCode));
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error, "AutoHangup Error: $ex");
    }
    int waitCount = 0;
    while (waitCount < 60) {
      if (!Storage.isConsultating.value) {
        break;
      }
      waitCount++;
      await Future.delayed(const Duration(milliseconds: 50));
    }
  }

  //检查FIS是否连接云服务
  Future fisCheck() async {
    try {
      var checkResult = await getUSMachineInfo();
      if (checkResult == null || !checkResult.isLogon) {
        checkResult = await getUSMachineInfo();
        if (checkResult == null || !checkResult.isLogon) {
          await loggerAsyn(FISDeviceLogCategory.Error, "FIS is Offline");
          await handleLogOutAsync();
        } else {
          _fisCheckTimer = Timer(const Duration(seconds: 3), () {
            fisCheck();
          });
        }
      } else {
        _fisCheckTimer = Timer(const Duration(seconds: 3), () {
          fisCheck();
        });
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error, 'FISCheckTimer error: $ex');
      await handleLogOutAsync();
    }
  }

  //检查FIS是否连接云服务
  Future<FISUSMachineInfoForFlutter?> getUSMachineInfo() async {
    try {
      return await jsonRpcProxyForFISLib.connection
          .getUSMachineInfoForFlutter();
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'getUSMachineInfo error: $ex');
      return null;
    }
  }

  Future<void> _informUserStatusAsync(bool isLogin) async {
    try {
      await jsonRpcProxyForFISLib.consultation.informUserStatusFormFlutter(
          FISUserLoginStatusArgs(
              status: isLogin
                  ? FISConsultationUserLoginStatus.Logon
                  : FISConsultationUserLoginStatus.Logoff,
              userId: isLogin ? _userCode : "",
              userName: isLogin ? _displayName : ""));
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'informUserStatusAsync error: $ex');
    }
  }

  Future<void> _getDeviceShortCode() async {
    try {
      var deviceInfo = await jsonRpcProxy.device.getDeviceInfoAsync(
          GetDeviceRequest(
              token: Storage.user.token, deviceCode: Storage.usDeviceCode));
      Storage.usDeviceShortCode = deviceInfo.shortCode;
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'getDeviceShortCode error: $ex');
    }
  }

  Future<void> _autoBindDeviceAsync() async {
    try {
      var userInfo = await jsonRpcProxy.user
          .getUserInfoAsync(GetUserInfoRequest(token: Storage.user.token));
      if (userInfo.bindDevices != null &&
          !userInfo.bindDevices!.contains(Storage.usDeviceCode)) {
        var bindResult = await jsonRpcProxy.device.scanBindDeviceAsync(
            ScanBindDeviceRequest(
                token: Storage.user.token,
                shortCode: Storage.usDeviceShortCode));
        if (!bindResult) {
          await loggerAsyn(FISDeviceLogCategory.Error,
              'autoBindDeviceAsync fail,scanBindDeviceAsync result is fail,token:${Storage.user.token},shortCode:${Storage.usDeviceShortCode}');
        }
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'autoBindDeviceAsync error: $ex');
    }
  }
}
