import 'package:fis_i18n/i18n.dart';
import 'package:fis_i18n/modules/index.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/inetworkerrorhandlermanager.dart';
import 'package:get/get.dart';

final _josnRpcClient = JsonRpcClient();
JsonRpcProxy get jsonRpcProxy => _josnRpcClient.getJsonRpcProxy();
JsonRpcClient get jsonrpcclient => _josnRpcClient;

class JsonRpcClient {
  late String host;
  late bool useSSL;
  JsonRpcProxy? _jsonRpcProxy;

  JsonRpcProxy getJsonRpcProxy() {
    if (_jsonRpcProxy == null) {
      throw Exception("JsonRpcProxy is null");
    }
    return _jsonRpcProxy!;
  }

  void updateServer(String host, [bool useSSL = false]) {
    if (_jsonRpcProxy == null) {
      _jsonRpcProxy = JsonRpcProxy(host: host);
      _jsonRpcProxy!.setServerHost(host, useSSL);
      _jsonRpcProxy!.addInterceptor(_DefaultInterceptor());
      _jsonRpcProxy!.addInterceptor(JsonRpcExInterceptor());
    } else if (this.host != host || this.useSSL != useSSL) {
      _jsonRpcProxy!.setServerHost(host, useSSL);
      _jsonRpcProxy!.clearCache();
    }
    this.host = host;
    this.useSSL = useSSL;
  }
}

class _DefaultInterceptor extends JsonRpcInterceptor {
  int _networkRetryCount = 0;

  @override
  Future<Map<String, dynamic>> onResponse(Map<String, dynamic> response) async {
    var networkErrorHandlerManager = Get.find<INetworkErrorHandlerManager>();
    networkErrorHandlerManager.onNetworkInterruption.emit('', false);
    _networkRetryCount = 0;
    return response;
  }

  @override
  Future<JsonRpcNetworkException> onHttpRequestError(
      JsonRpcNetworkException error) async {
    final String message = error.message ?? '';
    if (message.contains('FHttpError ') && _networkRetryCount == 0) {
      var networkErrorHandlerManager = Get.find<INetworkErrorHandlerManager>();
      _networkRetryCount += 1;
      networkErrorHandlerManager.onNetworkInterruption.emit('', true);
    }
    return error;
  }
}

class JsonRpcExInterceptor extends JsonRpcInterceptor {
  @override
  Future<JsonRpcServerError> onResponseError(JsonRpcServerError error) async {
    var errorCode = error.code;
    try {
      if (checkIsFISLibJsonRPCError(error)) {
        await loggerAsyn(FISDeviceLogCategory.Warn,
            "Fetch RPC error, code: $errorCode, msg: ${error.message}");
      } else {
        if (errorCode != null && errorCode != 1 && errorCode != 2) {
          String? text;
          if (errorCode == -1) {
            text = i18nBook
                .getTrItem(ErrorCodesModule.ModuleName, 'errorCodeUN')
                ?.t;
          } else {
            text = i18nBook
                .getTrItem(
                    ErrorCodesModule.ModuleName, 'errorCode${error.code}')
                ?.t;
          }
          if (errorCode != 6041) {
            PromptBox.toast(text ?? "Unkonwn RPC Exception");
          }
          await loggerAsyn(FISDeviceLogCategory.Error,
              "Fetch RPC error, code: ${error.code}, msg: ${error.message}");
        }
      }
    } catch (e) {
      PromptBox.toast("${i18nBook.errorCodes.errorCodeUN.t}: $errorCode");
      await loggerAsyn(FISDeviceLogCategory.Error,
          "Fetch RPC error, code: $errorCode, msg: ${error.message},ex:${e.toString()}");
    }
    return error;
  }

  bool checkIsFISLibJsonRPCError(JsonRpcServerError error) {
    try {
      if (error.code == null || error.message == null) {
        return false;
      }
      if (error.code! < 0 && error.message!.contains("FISLib")) {
        return true;
      }
      return false;
    } catch (ex) {
      return false;
    }
  }
}
