import 'dart:async';
import 'package:fis_common/extensions/string.dart';
import 'package:fis_common/helpers/encrypt.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_i18n/modules/error_codes.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';

class LoginController extends GetxController
    with GetSingleTickerProviderStateMixin {
  var loginBtnText = i18nBook.auth.login.t.obs;
  var userName = ''.obs;
  var password = ''.obs;
  var verifyCodeAccount = ''.obs;
  var verifyCode = ''.obs;
  var qrCodeExpired = false.obs;
  var isLoginWithVerificationCode = false.obs;
  var qrCode = "".obs;
  var isQrcode = false.obs;
  Timer? _timer;
  Timer? _expireTimer;

  /// tab选择器
  late TabController tabController;

  final TextEditingController userNameTextController = TextEditingController();
  final TextEditingController passwordTextController = TextEditingController();
  final TextEditingController verifyCodeAccountTextController =
      TextEditingController();
  final TextEditingController verifyCodeTextController =
      TextEditingController();

  /// 标签页列表
  List<String> get tabTitles => [
        i18nBook.auth.tabOfLoginByAccountPwd.t,
        i18nBook.auth.tabOfLoginByVerificationCode.t,
      ];

  @override
  void onInit() async {
    if (Storage.loginParameter.userName.isNotNullOrWhiteSpace) {
      userNameTextController.text = Storage.loginParameter.userName!;
      userName.value = Storage.loginParameter.userName!;
    }
    if (Storage.isRemeberPassword.value &&
        Storage.loginParameter.password.isNotNullOrWhiteSpace) {
      passwordTextController.text =
          FEncryptHelper.decryptBase64(Storage.loginParameter.password!);
      password.value = Storage.loginParameter.password!;
    }
    tabController = TabController(
      vsync: this,
      length: tabTitles.length,
      initialIndex: 0,
    );
    await startCheckQrcodeExpiredAsync();
    super.onInit();
  }

  /// 切换登录方式
  void onTabChanged(int index) {
    isLoginWithVerificationCode.value = index == 1;
  }

  void updateText(TextEditingController controller, String text) {
    if (controller == userNameTextController) {
      userName.value = text;
    } else if (controller == passwordTextController) {
      password.value = text;
    } else if (controller == verifyCodeAccountTextController) {
      verifyCodeAccount.value = text;
    } else if (controller == verifyCodeTextController) {
      verifyCode.value = text;
    }
  }

  @override
  void onClose() {
    super.onClose();
  }

  Future<bool> sendVerifyCodeAsync() async {
    var result = await getUSMachineInfoAsync();
    if (!result) {
      return false;
    }

    bool canSend = false;
    if (verifyCodeAccountTextController.text.isNotEmpty) {
      if (i18nBook.isCurrentChinese) {
        canSend = verifyCodeAccountTextController.text.isEmail ||
            verifyCodeAccountTextController.text.isChineseMobileNumber;
      } else {
        canSend = verifyCodeAccountTextController.text.isEmail;
      }
    }
    if (!canSend) {
      PromptBox.toast(i18nBook.auth.toast4AccountDouble.t);
      return false;
    }

    bool valid = false;
    final isMobileAccount =
        verifyCodeAccountTextController.text.isChineseMobileNumber;
    try {
      if (isMobileAccount) {
        valid = await jsonRpcProxy.login.sendSMSVerificationCodeAsync(
          SendSMSVerificationCodeRequest(
              userPhone: verifyCodeAccountTextController.text),
        );
      } else {
        valid = await jsonRpcProxy.login.sendEmailVerificationCodeAsync(
          SendEmailVerificationCodeRequest(
            emailAddress: verifyCodeAccountTextController.text,
            languageCode: i18nBook.isCurrentChinese ? 'zh-CN' : 'en-US',
          ),
        );
      }
      if (valid) {
        PromptBox.toast(i18nBook.auth.toast4VerifyCodeSendSuccess.t);
        return true;
      }
      return false;
    } on JsonRpcNetworkException catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'LoginController send ${isMobileAccount ? 'message' : 'email'} verify code error: $e');
      return false;
    } catch (e) {
      await handlerErrorAsync(e,
          'LoginController send ${isMobileAccount ? 'message' : 'email'} verify code error: $e');
      return false;
    }
  }

  Future loginAsync() async {
    try {
      loginBtnText.value = i18nBook.auth.logining.t;
      var result = await getUSMachineInfoAsync();
      if (!result) {
        return;
      }
      String password = "";
      String account = "";
      try {
        if (!isLoginWithVerificationCode.value) {
          password =
              FEncryptHelper.symmetryEncrypt(passwordTextController.text);
          account = userNameTextController.text.trim();
        } else {
          account = verifyCodeAccountTextController.text;
          if (!account.isChineseMobileNumber && !account.isEmail) {
            PromptBox.toast(i18nBook.auth.toast4AccountDouble.t);
            return;
          }
        }
      } on IndexError catch (ex) {
        PromptBox.toast(i18nBook.errorCodes.errorCode3042.t);
        await loggerAsyn(FISDeviceLogCategory.Error,
            "Password is illegal,the password is ${passwordTextController.text},ErrorMessage:$ex");
        return;
      }

      final loginResult = await jsonRpcProxy.login.commonLoginAsync(
        CommonLoginRequest(
          anyAccount: account,
          anyCode: isLoginWithVerificationCode.value
              ? verifyCodeTextController.text
              : '',
          password: isLoginWithVerificationCode.value
              ? ''
              : (passwordTextController.text.isEmpty ? '' : password),
          platform: Storage.platform,
          loginSource: LoginSource.US,
        ),
      );

      /// 密码输错之后 错误提示
      if (loginResult.lockRemainingTimes != null) {
        PromptBox.toast(i18nBook.auth.remainingTimes.translate(
          [loginResult.lockRemainingTimes],
        ));
        return;
      }

      if (loginResult.loginState == LoginStateEnum.Succeed) {
        if (loginResult.token != null && loginResult.token!.isNotEmpty) {
          _timer?.cancel();
          _expireTimer?.cancel();
          updateLoginParameter();

          await _handleLoginAsync(loginResult);
          Get.toNamed(RouteConstants.home);
        }
      }
    } on JsonRpcNetworkException {
      PromptBox.toast(i18nBook.auth.toast4LoginFailNetworkIssues.t);
    } catch (e) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, 'LoginController Login failed: $e');
    } finally {
      loginBtnText.value = i18nBook.auth.login.t;
    }
  }

  void updateLoginParameter() {
    if (!isLoginWithVerificationCode.value) {
      Storage.loginParameter.userName = userNameTextController.text;
      if (Storage.isRemeberPassword.value) {
        Storage.loginParameter.password =
            FEncryptHelper.encodeBase64(passwordTextController.text);
      } else {
        Storage.loginParameter.password = null;
      }
      Storage.updateLoginParameter();
    }
  }

  Future startCheckQrcodeExpiredAsync() async {
    var getScanCodeResult = await _getScanCodeAsync();
    if (getScanCodeResult == null) {
      qrCodeExpired.value = true;
      _timer?.cancel();
      _expireTimer?.cancel();
      return;
    }
    qrCode.value = getScanCodeResult.scanCode!;
    qrCodeExpired.value = false;
    _timer = Timer.periodic(const Duration(seconds: 1), (timer) async {
      try {
        var checkResult = await jsonRpcProxy.login.checkConfirmScanStateAsync(
            CheckConfirmScanStateRequest(scanCode: qrCode.value));
        switch (checkResult.checkConfirmScanState) {
          case CheckConfirmScanState.Wait:
            break;
          case CheckConfirmScanState.Success:
            var result = await getUSMachineInfoAsync();
            if (!result) {
              qrCodeExpired.value = true;
              _timer?.cancel();
              _expireTimer?.cancel();
              break;
            }
            _timer?.cancel();
            _expireTimer?.cancel();
            try {
              await _handleLoginAsync(checkResult.loginResult!);
              Get.toNamed(RouteConstants.home);
            } catch (e) {
              await loggerAsyn(FISDeviceLogCategory.Error,
                  'LoginController scan qr code login error: $e');
            }
            break;
          case CheckConfirmScanState.Expire:
            qrCodeExpired.value = true;
            _timer?.cancel();
            _expireTimer?.cancel();
            break;
          default:
        }
      } on JsonRpcNetworkException catch (ex) {
        await loggerAsyn(FISDeviceLogCategory.Error,
            'LoginController CheckQrcodeExpired timer error: $ex');
        qrCodeExpired.value = true;
        _timer?.cancel();
        _expireTimer?.cancel();
      } catch (ex) {
        await handlerErrorAsync(
            ex, 'LoginController CheckQrcodeExpired timer error: $ex');
        qrCodeExpired.value = true;
        _timer?.cancel();
        _expireTimer?.cancel();
      }
    });

    _expireTimer = Timer(Duration(seconds: getScanCodeResult.validSeconds), () {
      if (_timer != null && _timer!.isActive) {
        qrCodeExpired.value = true;
        _timer?.cancel();
      }
    });
  }

  Future<GetScanCodeResult?> _getScanCodeAsync() async {
    try {
      var result = await getUSMachineInfoAsync();
      if (!result) {
        return null;
      }
      var getResult =
          await jsonRpcProxy.login.getScanCodeAsync(GetScanCodeRequest(
        platform: Storage.platform,
        scanLoginSource: ScanLoginSource.US,
      ));
      return getResult;
    } on JsonRpcNetworkException catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'LoginController getScanCodeAsync Errorr: $e');
      return null;
    } catch (e) {
      await handlerErrorAsync(e, 'LoginController getScanCodeAsync Errorr: $e');
      return null;
    }
  }

  Future<void> _handleLoginAsync(LoginResult result) async {
    try {
      await _saveLoginInStateAsync(result);
    } on JsonRpcNetworkException catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'LoginController _handleLoginAsync errorr: $e');
    } catch (e) {
      await handlerErrorAsync(
          e, 'LoginController _handleLoginAsync errorr: $e');
    }
  }

  /// 保存登录状态
  Future<void> _saveLoginInStateAsync(LoginResult info) async {
    await Storage.user.handleLoginAsync(
        token: info.token!, isPasswordExpired: info.passwordExpired);
    await Storage.user.handleAfterLoginAsync();
  }

  Future<bool> getUSMachineInfoAsync() async {
    try {
      var usMachineInfo =
          await jsonRpcProxyForFISLib.connection.getUSMachineInfoForFlutter();
      if (usMachineInfo.serverUrl.isNotNullOrEmpty && usMachineInfo.isLogon) {
        jsonrpcclient.updateServer(
            usMachineInfo.serverUrl!, usMachineInfo.useSSL);
      } else {
        PromptBox.toast(fisI18nBook.common.pleaseConnectVCloudFirst.t);
        return false;
      }
      Storage.usDeviceCode = usMachineInfo.deviceCode;
      return true;
    } on JsonRpcNetworkException catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'LoginController getUSMachineInfoForFlutter Errorr: $e');
      return false;
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'LoginController getUSMachineInfoForFlutter Errorr: $ex');
      return false;
    }
  }
}
