import 'dart:convert';
import 'package:fis_common/extensions/string.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/basemanager.dart';
import 'package:flyinsonolite/managers/interfaces/iconfigurationmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/services/sharedstorageservice.dart';
import 'package:get/get.dart';

///存储key的关键字静态词典
class StoreKeys {
  static const consultationConfig = "ConsultationConfig";

  ///配置存储Key
  static const consultationConfigVersion = "ConsultationConfigVersion";

  static const languageConfig = "LanguageConfig";

  static const reportEditSetting = "ReportEditSetting";

  static const loginConfig = "LoginConfig";

  static const remeberPwdConfig = "RemeberPwdConfig";
}

class ConfigurationManager extends BaseManager
    implements IConfigurationManager {
  ///检查更新
  ///判断是否更新本地缓存
  ///更新会诊字段，病人自动和对应翻译
  @override
  Future checkUpdateAsync() async {
    try {
      if (Storage.configurationFromerver == null) {
        var consultationServerConfig =
            await _getConsultationServerConfigAsync();
        var languageConfig = await _getLangugeDataAsync();
        var configurationFromerver =
            ConfigurationFromServer(consultationServerConfig!, languageConfig!);
        Storage.configurationFromerver = configurationFromerver;
      }
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConfigurationService checkUpdateAsync error $e.");
    }
  }

  /// 获取翻译包
  Future<LanguageServerConfig?> _getLangugeDataAsync() async {
    LanguageServerConfig? languageConfig;
    try {
      var languageConfigCacheString =
          SharedStorageService.ins.getString(StoreKeys.languageConfig);
      if (languageConfigCacheString.isNotNullOrEmpty) {
        languageConfig = LanguageServerConfig.fromJson(
            jsonDecode(languageConfigCacheString!));
      }
      var currentLangugeKey = FI18n.ins.isCurrentChinese ? 'zh-CN' : 'en-US';
      if (languageConfig?.languageKey == null ||
          languageConfig?.languageKey != currentLangugeKey) {
        languageConfig = null;
      }
      final result = await jsonRpcProxy.organization.getLangugeDataListAsync(
        LangugeDataRequest(
            token: token,
            langugeKey: FI18n.ins.isCurrentChinese ? 'zh-CN' : 'en-US',
            langugeDataItemList:
                _getLangugeDataItemRequestList(languageConfig)),
      );

      if (languageConfig == null) {
        languageConfig = LanguageServerConfig(result, currentLangugeKey);
      } else {
        for (var item in result) {
          if (item.content.isNullOrWhiteSpace) {
            continue;
          }

          var existItem = languageConfig.languageList.firstWhereOrNull(
              (element) => element.langugeModule == item.langugeModule);
          if (existItem == null) {
            languageConfig.languageList.add(item);
          } else {
            existItem.content = item.content;
            existItem.version = item.version;
          }
        }
      }
      await SharedStorageService.ins
          .setString(StoreKeys.languageConfig, jsonEncode(languageConfig));
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConfigurationManager _getLangugeDataAsync error: $e");
    }

    return languageConfig;
  }

  List<LangugeDataItemRequest> _getLangugeDataItemRequestList(
      LanguageServerConfig? languageServerConfig) {
    List<LangugeDataItemRequest> langugeDataItemRequestList = [];
    if (languageServerConfig != null) {
      for (var element in languageServerConfig.languageList) {
        langugeDataItemRequestList.add(LangugeDataItemRequest(
            langugeModule: element.langugeModule, version: element.version));
      }
    }

    return langugeDataItemRequestList;
  }

  Future<ConsultationServerConfig?> _getConsultationServerConfigAsync() async {
    ConsultationServerConfig? consultationServerConfig;
    try {
      var consultationConfigCacheString =
          SharedStorageService.ins.getString(StoreKeys.consultationConfig);
      var consultationConfigVersion = SharedStorageService.ins
              .getString(StoreKeys.consultationConfigVersion) ??
          "0.9";

      if (consultationConfigCacheString.isNotNullOrEmpty) {
        consultationServerConfig = ConsultationServerConfig.fromJson(
            jsonDecode(consultationConfigCacheString!)['ConsultationInfo'],
            consultationConfigVersion);
        consultationConfigVersion = consultationServerConfig.version;
      }

      final consultationConfig =
          await jsonRpcProxy.organization.getOrganizationSettingAsync(
        GetUserOrganizationSettingRequest(
          version: consultationConfigVersion,
          token: token,
          settingType: OrganizationSettingTypeEnum.Consultation,
        ),
      );
      if (consultationConfig.settingVersion != consultationConfigVersion &&
          consultationConfig.settingData != null) {
        var jsonObject = jsonDecode(consultationConfig.settingData!);
        consultationServerConfig = ConsultationServerConfig.fromJson(
            jsonObject['ConsultationInfo'], consultationConfig.settingVersion!);
        await SharedStorageService.ins.setString(
            StoreKeys.consultationConfig, consultationConfig.settingData!);
        await SharedStorageService.ins.setString(
            StoreKeys.consultationConfigVersion,
            consultationConfig.settingVersion!);
      }
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConfigurationManager _getConsultationServerConfigAsync error $e.");
    }

    return consultationServerConfig;
  }
}
